VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "ContainerLayout"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' ====== Reusable columns layout =========
Option Explicit

Private container_ As Visio.Shape

Private margins_ As Double
Private headerSize_ As Double
Private maxHeight_ As Double

Private columnWidth_ As Double
Private lastColumn_ As Collection
Private originX_ As Double
Private originY_ As Double
Private currentY_ As Double
Private currentX_ As Double

Public Function Init(target As Visio.Shape, maxHeight As Double)
  Set container_ = target
  maxHeight_ = maxHeight
  
  margins_ = container_.CellsU(CELLSU_CONTAINER_MARGINS) + LM_MARGINS_EPSILON
  headerSize_ = container_.Shapes(2).CellsU("Height")
  
  Call container_.Resize(visResizeDirS, container_.Shapes(2).CellsU("Height") + 2 * margins_ - container_.CellsU("Height"), visInches)
  originY_ = container_.CellsU("PinY") + container_.CellsU("Height") / 2#
  currentY_ = originY_ - container_.Shapes(2).CellsU("Height") - margins_
  
  originX_ = container_.CellsU("PinX") - container_.CellsU("Width") / 2#
  currentX_ = originX_ + margins_
  
  columnWidth_ = 0
  Set lastColumn_ = New Collection
End Function

Public Property Get AreaHeight() As Double
  AreaHeight = container_.CellsU("Height") - 2 * margins_ - headerSize_
End Property

Public Property Get AreaWidth() As Double
  AreaWidth = container_.CellsU("Width") - 2 * margins_
End Property

Public Function AddItem(target As Visio.Shape)
  If Not CanAdd(target) Then _
    Call FinalizeColumn
  Call AddShapeInternal(target)
End Function

Public Function AddItemToNewColumn(target As Visio.Shape)
  Call FinalizeColumn
  Call AddShapeInternal(target)
End Function

Public Function Finalize()
  Call FinalizeColumn
End Function

' ================
Private Function CanAdd(target As Visio.Shape) As Boolean
  If lastColumn_.Count = 0 Then
    CanAdd = False
    Exit Function
  End If
  
  Dim newHeight As Double: newHeight = Abs(originY_ - currentY_) + target.CellsU("Height") + LM_VERTICAL_GAP
  If Round(newHeight + margins_, DEPSILON) < Round(container_.CellsU("Height"), DEPSILON) Then
    CanAdd = True
    Exit Function
  End If
    
  CanAdd = Round(newHeight + margins_, DEPSILON) < Round(maxHeight_, DEPSILON)
End Function

Private Function AddShapeInternal(target As Visio.Shape)
  Dim newHeight As Double: newHeight = Abs(originY_ - currentY_) + target.CellsU("Height")
  Call EnsureHeight(newHeight + margins_)
  
  Dim newWidth As Double: newWidth = Abs(originX_ - currentX_) + target.CellsU("Width")
  Call EnsureWidth(newWidth + margins_)
  
  Call target.SetCenter(originX_ + newWidth - target.CellsU("Width") / 2#, _
                        originY_ - newHeight + target.CellsU("Height") / 2#)
  currentY_ = originY_ - newHeight - LM_VERTICAL_GAP
  If Round(target.CellsU("Width"), DEPSILON) > Round(columnWidth_, DEPSILON) Then _
    columnWidth_ = target.CellsU("Width")
  
  Call lastColumn_.Add(target)
End Function

Private Function EnsureHeight(newHeight As Double)
  If Round(newHeight, DEPSILON) < Round(container_.CellsU("Height"), DEPSILON) Then _
    Exit Function
  Call container_.Resize(visResizeDirS, newHeight - container_.CellsU("Height"), visInches)
End Function

Private Function EnsureWidth(newWidth As Double)
  If Round(newWidth, DEPSILON) < Round(container_.CellsU("Width"), DEPSILON) Then _
    Exit Function
  Call container_.Resize(visResizeDirE, newWidth - container_.CellsU("Width"), visInches)
End Function

Private Function FinalizeColumn()
  If lastColumn_.Count = 0 Then _
    Exit Function

  Call EqualizeWidth
  
  currentX_ = currentX_ + columnWidth_ + LM_HORIZONTAL_GAP
  currentY_ = originY_ - container_.Shapes(2).CellsU("Height") - margins_
  columnWidth_ = 0
  Set lastColumn_ = New Collection
End Function

Private Function EqualizeWidth()
  Dim nItem&
  Dim aShape As Visio.Shape
  For nItem = 1 To lastColumn_.Count Step 1
    Set aShape = lastColumn_.Item(nItem)
    If Round(columnWidth_, DEPSILON) > Round(aShape.CellsU("Width"), DEPSILON) Then
      Call aShape.Resize(visResizeDirE, columnWidth_ - aShape.CellsU("Width"), visInches)
    End If
  Next nItem
End Function
