VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "LayoutManager"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' ====== Shape layout and combination logic ========
Option Explicit

Private data_ As CDS_StaticHierarchy
Private storage_ As API_ShapeStorage

Private parentPage_ As Visio.Page

Private pageProps_ As PageProps
Private originX_ As Double
Private originY_ As Double

Public Function Init(ByRef theData As CDS_StaticHierarchy, ByRef iStorage As API_ShapeStorage, originX As Double, originY As Double)
  Set data_ = theData
  Set storage_ = iStorage
  Set parentPage_ = iStorage.TargetPage
  originX_ = originX
  originY_ = originY
End Function

Public Function CombineContainers(props As PageProps)
  pageProps_ = props
  Dim iNode As CDS_NodeSH
  For Each iNode In data_.nodes_
    If iNode.rank_ = 1 Then _
      Call PrepareItem(iNode, props.frontHeight_)
    Call CSE_ProgressBar.IncrementA
  Next iNode
  Call ActiveWindow.Selection.DeselectAll
End Function

Public Function Build(props As PageProps)
  Dim iPage As New PageLayout: Call iPage.Init(parentPage_, props)
  Dim itemShape As Visio.Shape
  Dim iNode As CDS_NodeSH
  For Each iNode In data_.nodes_
    If iNode.rank_ = 1 Then
      Set itemShape = iNode.data_.shape_
      Call storage_.GiveBack(itemShape)
      Call iPage.AddItem(itemShape)
      Call CSE_ProgressBar.IncrementA
    End If
  Next iNode
  Call iPage.FinalizePage
End Function

' =======
Private Function PrepareItem(target As CDS_NodeSH, dMaxHeight As Double) ' recursive
  If target.children_.Count = 0 Then _
    Exit Function
  Select Case target.data_.type_
    Case T_ITEM_FRONT: Call PrepareFront(target)
    Case T_ITEM_PROBLEM: Call PrepareProblem(target, pageProps_.problemHeight_)
    Case T_ITEM_SUBJECT_TYPE, T_ITEM_GROUP: Call PrepareContainer(target, dMaxHeight)
    Case T_ITEM_ORGAN: Call PrepareOrgan(target)
    Case Else: Call Err.Raise(1000, Description:="Structural error in " & target.data_.text_)  ' TODO: specify err number
  End Select
End Function

Private Function PrepareChildren(target As CDS_NodeSH, dMaxHeight As Double)
  Dim iChildNode As CDS_NodeSH
  For Each iChildNode In target.children_
    Call PrepareItem(iChildNode, dMaxHeight)
  Next iChildNode
End Function

Private Function PrepareFront(target As CDS_NodeSH)
  Dim dHeight As Double: dHeight = pageProps_.frontHeight_
  If target.descendantsCount_ > HEIGHT_TRIPLE_THRESHOLD Then
    dHeight = dHeight * 3
  ElseIf target.descendantsCount_ > HEIGHT_DOUBLE_THRESHOLD Then
    dHeight = dHeight * 2
  End If
  Dim iLayout As ContainerLayout
  Set iLayout = CreateLayout(target, dHeight)
  Call storage_.GiveBack(target.data_.shape_)
  Call IncludeFrontChildren(target, iLayout)
  Call iLayout.Finalize
  Call storage_.Store(target.data_.shape_)
End Function

Private Function PrepareProblem(target As CDS_NodeSH, dMaxHeight As Double)
  Dim iLayout As ContainerLayout
  Set iLayout = CreateLayout(target, dMaxHeight)
  Call storage_.GiveBack(target.data_.shape_)
  Call IncludeProblemChildren(target, iLayout)
  Call iLayout.Finalize
  Call storage_.Store(target.data_.shape_)
End Function

Private Function PrepareContainer(target As CDS_NodeSH, dMaxHeight As Double)
  Dim iLayout As ContainerLayout
  Set iLayout = CreateLayout(target, dMaxHeight)
  Call storage_.GiveBack(target.data_.shape_)
  Call IncludeContainerChildren(target, iLayout)
  Call iLayout.Finalize
  Call storage_.Store(target.data_.shape_)
End Function

Private Function PrepareOrgan(target As CDS_NodeSH)
  Call storage_.GiveBack(target.data_.shape_)
  Set target.data_.shape_ = GroupWithChildren(target)
  Call storage_.Store(target.data_.shape_)
End Function

Private Function CreateLayout(target As CDS_NodeSH, dMaxHeight As Double) As ContainerLayout
  Dim iParent As Visio.Shape: Set iParent = target.data_.shape_
  Dim newMaxHeight As Double: newMaxHeight = dMaxHeight - HeightBoilerplate(iParent)
  Call PrepareChildren(target, newMaxHeight)
  
  Call iParent.SetCenter(originX_ + iParent.CellsU("Width") / 2#, originY_ - iParent.CellsU("Height") / 2#)
  Dim aLayout As New ContainerLayout: Call aLayout.Init(iParent, dMaxHeight)
  Set CreateLayout = aLayout
End Function

Private Function IncludeFrontChildren(target As CDS_NodeSH, iLayout As ContainerLayout)
  Dim iChildNode As CDS_NodeSH
  For Each iChildNode In target.children_
    Dim iChild As Visio.Shape: Set iChild = iChildNode.data_.shape_
    Call storage_.GiveBack(iChild)
    Call iLayout.AddItem(iChild)
  Next iChildNode
End Function

Private Function IncludeProblemChildren(target As CDS_NodeSH, iLayout As ContainerLayout)
  Dim iChildNode As CDS_NodeSH
  For Each iChildNode In target.children_
    Dim iChild As Visio.Shape: Set iChild = iChildNode.data_.shape_
    Call storage_.GiveBack(iChild)
    Call iLayout.AddItemToNewColumn(iChild)
  Next iChildNode
  
  For Each iChildNode In target.children_
    Call iChildNode.data_.shape_.Resize(visResizeDirS, iLayout.AreaHeight - iChildNode.data_.shape_.CellsU("Height"), visInches)
  Next iChildNode
End Function

Private Function IncludeContainerChildren(target As CDS_NodeSH, iLayout As ContainerLayout)
  Dim nLastType As TItem: nLastType = T_ITEM_PROBLEM
  Dim iChildNode As CDS_NodeSH
  For Each iChildNode In target.children_
    Dim iChild As Visio.Shape: Set iChild = iChildNode.data_.shape_
    Call storage_.GiveBack(iChild)
    If nLastType = iChildNode.data_.type_ Then
      Call iLayout.AddItem(iChild)
    Else
      Call iLayout.AddItemToNewColumn(iChild)
    End If
    nLastType = iChildNode.data_.type_
  Next iChildNode
End Function

Private Function GroupWithChildren(target As CDS_NodeSH) As Visio.Shape
  Dim iParent As Visio.Shape: Set iParent = target.data_.shape_
  Call iParent.SetCenter(originX_ + iParent.CellsU("Width") / 2#, originY_ - iParent.CellsU("Height") / 2#)
  
  Dim groupSel As Visio.Selection: Set groupSel = parentPage_.CreateSelection(visSelTypeEmpty)
  Call groupSel.Select(iParent, visSelect)
  
  Dim posY As Double: posY = originY_ - iParent.CellsU("Height")
  Dim maxWidth As Double: maxWidth = iParent.CellsU("Width")
  
  Dim iChildNode As CDS_NodeSH
  For Each iChildNode In target.children_
    Dim iChild As Visio.Shape: Set iChild = iChildNode.data_.shape_
    Call storage_.GiveBack(iChild)
    Call groupSel.Select(iChild, visSelect)
    posY = posY - iChild.CellsU("Height")
    Call iChild.SetCenter(originX_ + iChild.CellsU("Width") / 2#, posY + iChild.CellsU("Height") / 2#)
    If VBA.Round(maxWidth, DEPSILON) < VBA.Round(iChild.CellsU("Width"), DEPSILON) Then _
      maxWidth = iChild.CellsU("Width")
  Next iChildNode
  
  Dim aShape As Visio.Shape
  For Each aShape In groupSel
    If VBA.Round(maxWidth, DEPSILON) > VBA.Round(aShape.CellsU("Width"), DEPSILON) Then
      Call aShape.Resize(visResizeDirE, maxWidth - aShape.CellsU("Width"), visInches)
    End If
  Next aShape
  
  Set GroupWithChildren = groupSel.Group
End Function

Private Function HeightBoilerplate(target As Visio.Shape) As Double
  If target.Shapes.Count < 2 Then
    HeightBoilerplate = 0
    Exit Function
  End If
  HeightBoilerplate = target.Shapes(2).CellsU("Height")
  HeightBoilerplate = HeightBoilerplate + 2 * (target.CellsU(CELLSU_CONTAINER_MARGINS) + LM_MARGINS_EPSILON)
End Function
