Attribute VB_Name = "Main"
Option Explicit

Public Sub RunGenerator()
  Dim thePage As Visio.Page: Set thePage = MainPage()
  Call ImportDialog.Init(thePage.PageSheet.CellsU("PageHeight").Result(visMillimeters))
  Call ImportDialog.Show
  If ImportDialog.isCancelled_ Then _
    Exit Sub
  
  Dim theData As CDS_StaticHierarchy: Set theData = ExtractData(ImportDialog.FileName)
  If theData Is Nothing Then _
    Exit Sub
  
  If theData.nodes_.Count = 0 Then
    Call UserInteraction.ShowMessage(EM_DATA_EMPTY)
    Exit Sub
  End If
  
  Call GlobalUndo.BeginScope("")
  Call VsoClearPage(MainPage())
  Call GlobalUndo.EndScope
  
  Call GlobalUndo.BeginScope(" ")
  Call CSE_ProgressBar.Init(" ", maxVal:=theData.nodes_.Count, curVal:=1)
  Call CSE_ProgressBar.ShowModeless
  
  Dim iStorage As API_ShapeStorage
  Set iStorage = CreateShapesFor(theData, MainPage())
  
  Dim iManager As New LayoutManager
  Call iManager.Init(theData, iStorage, originX:=0, originY:=thePage.PageSheet.CellsU("PageHeight"))
  
  ThisDocument.DiagramServicesEnabled = visServiceAll
  
  Call CSE_ProgressBar.Init(" ", maxVal:=theData.nodes_.Count, curVal:=1)
   Call iManager.CombineContainers(ImportDialog.GetProps)
  
  Call CSE_ProgressBar.Init(" ", maxVal:=iStorage.items_.Count, curVal:=1)
  Call iManager.Build(ImportDialog.GetProps)
  
  ThisDocument.DiagramServicesEnabled = visServiceNone
  
  Call Unload(CSE_ProgressBar)
  Call GlobalUndo.EndScope
  
  Call VsoShowWholePage(thePage)
  Call UserInteraction.ShowMessage(IM_GENERATION_OK, _
                                    theData.nodes_.Count, _
                                    VBA.Round(thePage.PageSheet.CellsU("PageHeight").Result(visMillimeters), 0), _
                                    VBA.Round(thePage.PageSheet.CellsU("PageWidth").Result(visMillimeters), 0))
End Sub

Public Sub ClearMain()
  Call VsoClearPage(MainPage)
End Sub

Public Sub DeleteAllPages()
  Dim thePage As Visio.Page: Set thePage = MainPage()
  Dim target As Visio.Page
  Dim nPage&: nPage = 1
  Do While nPage <= ThisDocument.Pages.Count
    Set target = ThisDocument.Pages(nPage)
    If target.Name = thePage.Name Then
      nPage = nPage + 1
    Else
      Call target.Delete(True)
    End If
  Loop
End Sub

' ====
Private Function MainPage() As Visio.Page
  Set MainPage = Application.ActiveDocument.Pages(1)
End Function

Private Function ExtractData(targetPath$) As CDS_StaticHierarchy
  Dim wrap As New API_XLWrapper
  If wrap.OpenDocument(targetPath, bReadOnly:=True) Is Nothing Then _
    Exit Function
  
  Call CSE_ProgressBar.Init(" ", maxVal:=wrap.Document.Worksheets(1).UsedRange.Rows.Count, curVal:=2)
  Call CSE_ProgressBar.ShowModeless
  
  On Error Resume Next
  Set ExtractData = ScanExcel(wrap.Document.Worksheets(1))
  Call ProcessErrorMessages(UniqueList(ERR_XL_INVALID_TYPE))
  On Error GoTo 0
  
  Call Unload(CSE_ProgressBar)
  
  If ExtractData Is Nothing Then _
    Call UserInteraction.ShowMessage(EM_INVALID_FILE_FORMAT)
    
  Call wrap.ReleaseDocument
End Function
