VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "PageLayout"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private page_ As Visio.Page
Private props_ As PageProps

Private originX_ As Double
Private originY_ As Double

Private columnWidth_ As Double
Private lastColumn_ As Collection
Private columnCount_ As Long

Private currentY_ As Double
Private currentX_ As Double

Public Function Init(ByRef target As Visio.Page, props As PageProps)
  props_ = props
  Set page_ = target

  Set lastColumn_ = New Collection
  columnCount_ = 0
  columnWidth_ = 0
  originX_ = 0
  originY_ = props_.height_
  
  target.PageSheet.CellsU("PageHeight") = props.height_
  
  currentX_ = originX_ + props_.leftGap_
  currentY_ = originY_ - PAGE_VERTICAL_GAP
  If props.addHeader_ Then
    Call AddTitle
    currentY_ = originY_ - PAGE_TITLE_GAP
  End If
End Function

Public Function AddItem(target As Visio.Shape)
  If Not CanAdd(target) Then _
    Call FinalizeColumn
  Call AddShapeInternal(target)
End Function

Public Function AddItemToNewColumn(target As Visio.Shape)
  Call FinalizeColumn
  Call AddShapeInternal(target)
End Function

Public Function FinalizePage()
  Call FinalizeColumn
  If Not props_.multipage_ Then _
    page_.PageSheet.CellsU("PageWidth") = currentX_
End Function

' ================
Private Function CanAdd(target As Visio.Shape) As Boolean
  Dim newHeight As Double: newHeight = Abs(originY_ - currentY_) + target.CellsU("Height") + PAGE_VERTICAL_GAP
  CanAdd = Round(newHeight, DEPSILON) < Round(props_.height_, DEPSILON)
End Function

Private Function AddShapeInternal(target As Visio.Shape)
  Dim newHeight As Double: newHeight = Abs(originY_ - currentY_) + target.CellsU("Height")
  Dim newWidth As Double: newWidth = Abs(originX_ - currentX_) + target.CellsU("Width")
  
  Call target.SetCenter(originX_ + newWidth - target.CellsU("Width") / 2#, _
                        originY_ - newHeight + target.CellsU("Height") / 2#)
  currentY_ = originY_ - newHeight - PAGE_VERTICAL_GAP
  If Round(target.CellsU("Width"), DEPSILON) > Round(columnWidth_, DEPSILON) Then _
    columnWidth_ = target.CellsU("Width")
  
  Call lastColumn_.Add(target)
End Function

Private Function FinalizeColumn()
  If lastColumn_.Count = 0 Then _
    Exit Function
  
  columnCount_ = columnCount_ + 1
  ' Call EqualizeWidth ' Note: do not equalize width for fronts
  
  If props_.multipage_ Then
    Call CopyColumnToNewPage
    currentX_ = originX_ + props_.leftGap_
  ElseIf props_.width_ = 0 Then
    currentX_ = currentX_ + columnWidth_ + props_.columnGap_
  Else
    Dim nPageCount&: nPageCount = 1 + Int(currentX_ / props_.width_)
    Dim pageEdge As Double: pageEdge = originX_ + nPageCount * props_.width_
    
    If columnCount_ > 1 And VBA.Round(pageEdge, DEPSILON) < VBA.Round(currentX_ + columnWidth_ + props_.leftGap_, DEPSILON) Then
      Call AdjustLastColumnX(pageEdge - currentX_)
      currentX_ = pageEdge + columnWidth_ + props_.columnGap_
    Else
      currentX_ = currentX_ + columnWidth_ + props_.columnGap_
    End If
  End If
  
  currentY_ = originY_ - PAGE_VERTICAL_GAP
  columnWidth_ = 0
  Set lastColumn_ = New Collection
End Function

Private Function EqualizeWidth()
  Dim nItem&
  Dim aShape As Visio.Shape
  For nItem = 1 To lastColumn_.Count Step 1
    Set aShape = lastColumn_.Item(nItem)
    If Round(columnWidth_, DEPSILON) > Round(aShape.CellsU("Width"), DEPSILON) Then
      Call aShape.Resize(visResizeDirE, columnWidth_ - aShape.CellsU("Width"), visInches)
    End If
  Next nItem
End Function

Private Function AddTitle()
  Dim titleShape As Visio.Shape: Set titleShape = page_.Drop(FindMaster(page_.Document, SHP_TITLE), PAGE_HORIZONTAL_GAP, props_.height_ - PAGE_VERTICAL_GAP)
  titleShape.CellsU("PinY") = props_.height_ - PAGE_VERTICAL_GAP - titleShape.CellsU("Height") / 2#
  titleShape.CellsU("PinX") = PAGE_HORIZONTAL_GAP + titleShape.CellsU("Width") / 2#
  Call lastColumn_.Add(titleShape)
End Function

Private Function AdjustLastColumnX(xDiff As Double)
  Dim nItem&
  Dim aShape As Visio.Shape
  For nItem = 1 To lastColumn_.Count Step 1
    Set aShape = lastColumn_.Item(nItem)
    aShape.Cells("PinX") = aShape.Cells("PinX") + xDiff
  Next nItem
End Function

Private Function CopyColumnToNewPage()
  Dim newPage As Visio.Page:  Set newPage = page_.Document.Pages.Add
  newPage.PageSheet.CellsU("PageHeight") = page_.PageSheet.CellsU("PageHeight")
  If props_.width_ = 0 Then
    newPage.PageSheet.CellsU("PageWidth") = currentX_ + columnWidth_
  Else
    newPage.PageSheet.CellsU("PageWidth") = props_.width_
  End If
  
  Call SetupFixedGridPage(newPage)
  
  Dim aShape As Visio.Shape
  For Each aShape In lastColumn_
    Call aShape.Copy(visCopyPasteNoTranslate + visCopyPasteDontAddToContainers + visCopyPasteNoCascade)
    Call newPage.Paste(visCopyPasteNoTranslate + visCopyPasteDontAddToContainers + visCopyPasteNoCascade)
    
    Call aShape.DeleteEx(visDeleteNormal)
  Next aShape
End Function
